__REDIS_URL_KEY = "REDIS_URL"


def redis_decode(value):
    if isinstance(value, bytes):
        return value.decode('utf8')
    elif isinstance(value, (list, tuple, set)):
        return type(value)(map(redis_decode, value))
    else:
        return value


def redis_connect(url=None):
    if not url:
        from .zconfig import get_param
        url = get_param(__REDIS_URL_KEY)
    assert url
    return ZRedis(url)


class ZRedis:
    def __init__(self, url):
        import redis
        self.client = redis.from_url(url)

    def bpop_log(self, key, wait_log=None):
        if self.llen(key) <= 0:
            if wait_log:
                print(wait_log)
        return self.blpop(key)

    def get(self, key):
        return redis_decode(self.client.get(key))

    def set(self, key, value):
        return self.client.set(key, value)

    def expire(self, key, ttl):
        return self.client.expire(key, ttl)

    def set_expire(self, key, value, ttl):
        self.set(key, value)
        return self.expire(key, ttl)

    def rename(self, src, dst):
        self.client.rename(src, dst)

    def sall(self, key):
        return [redis_decode(it) for it in self.client.sunion(key)]

    def sadd(self, key, *value):
        return self.client.sadd(key, *value)

    def sismember(self, key, value):
        return self.client.sismember(key, value)

    def keys(self, pattern='*'):
        return list(map(redis_decode, self.client.keys(pattern)))

    def sdiff(self, keys, *args):
        return self.client.sdiff(keys, *args)

    def delete(self, *keys):
        return self.client.delete(*keys)

    def lpush(self, key, *values):
        return self.client.lpush(key, *values)

    def llen(self, key):
        return self.client.llen(key)

    def blpop(self, keys, timeout=0):
        return redis_decode(self.client.blpop(keys, timeout))[-1]

    def brpoplpush(self, src, dst, timeout=0):
        return redis_decode(self.client.brpoplpush(src, dst, timeout))

    def lpop(self, key):
        return redis_decode(self.client.lpop(key))

    def rpop(self, key):
        return redis_decode(self.client.rpop(key))

    def brpop(self, keys, timeout=0):
        return redis_decode(self.client.brpop(keys, timeout))[-1]

    def lall(self, key):
        return [redis_decode(it) for it in self.client.lrange(key, 0, -1)]

    def lrem(self, key, value):
        self.client.lrem(key, 0, value)

    def lpopall(self, key):
        items = [redis_decode(it) for it in self.client.lrange(key, 0, -1)]
        self.delete(key)
        return items

    def rpush(self, key, *values):
        return self.client.rpush(key, *values)



import unittest
from zzpy import randint


class ZRandomRandintTests(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_randint_when_no_parameters(self):
        for _ in range(10_000):
            n = randint()
            self.assertTrue(0 <= n <= 9)

    def test_randint_when_only_given_min(self):
        for _ in range(10_000):
            n = randint(min=5)
            self.assertTrue(5 <= n <= 9)

    def test_randint_when_only_given_max(self):
        for _ in range(10_000):
            n = randint(max=5)
            self.assertTrue(0 <= n <= 5)

    def test_randint_when_given_min_and_max(self):
        for _ in range(10_000):
            n = randint(min=1, max=5)
            self.assertTrue(1 <= n <= 5)

    def test_randint_when_min_equals_max(self):
        for _ in range(10_000):
            n = randint(min=1, max=1)
            self.assertEqual(n, 1)

    def test_randint_when_min_is_larger_then_max(self):
        for _ in range(10_000):
            n = randint(min=5, max=1)
            self.assertTrue(1 <= n <= 5)



import json
import datetime


class DateEncoder(json.JSONEncoder):
    """
    Json 无法解析 datatime 类型的数据，构建 DateEncoder 类解决 datatime 解析问题
    https://blog.csdn.net/yilovexing/article/details/93633436
    """

    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime("%Y-%m-%d %H:%M:%S")
        else:
            return super.default(obj)
            # return json.JSONEncoder.default(self, obj)


def jsondumps(obj, *, skipkeys=False, ensure_ascii=True, check_circular=True,
              allow_nan=True, cls=None, indent=None, separators=None,
              default=None, sort_keys=False, **kw):
    if not cls:
        cls = DateEncoder
    return json.dumps(obj, skipkeys=skipkeys, ensure_ascii=ensure_ascii, check_circular=check_circular,
                      allow_nan=allow_nan, cls=cls, indent=indent, separators=separators,
                      default=default, sort_keys=sort_keys, **kw)


def main():
    print(jsondumps({"k": "v"}, indent=4))


if __name__ == "__main__":
    main()



import unittest
import zzpy as z


def get_month(months=0, separator='-'):
    import arrow
    return arrow.now().shift(months=months).format(f"YYYY{separator}MM")


class TestCase(unittest.TestCase):
    def test_get_month_with_no_parameters(self):
        self.assertEqual(z.get_month(), get_month())

    def test_get_month_with_days(self):
        for offset in range(-1000, 1000):
            self.assertEqual(z.get_month(offset), get_month(offset))
            self.assertEqual(z.get_month(months=offset),
                             get_month(months=offset))
            self.assertEqual(z.get_month(offset), get_month(offset))
            self.assertEqual(z.get_month(months=offset),
                             get_month(months=offset))

    def test_get_month_with_separator(self):
        self.assertEqual(z.get_month(), get_month(separator="-"))
        self.assertEqual(z.get_month(), z.get_month(separator="-"))
        self.assertEqual(z.get_month(separator="-"), get_month(separator="-"))
        self.assertEqual(z.get_month(separator="/"), get_month(separator="/"))

    def test_get_month_with_days_and_separator(self):
        self.assertEqual(z.get_month(), get_month(months=0, separator="-"))
        self.assertEqual(z.get_month(), z.get_month(months=0, separator="-"))
        self.assertEqual(z.get_month(months=1, separator="*"),
                         get_month(months=1, separator="*"))
        self.assertEqual(z.get_month(months=1, separator="*"),
                         z.get_month(months=1, separator="*"))
        self.assertEqual(z.get_month(months=-1, separator="*"),
                         get_month(months=-1, separator="*"))
        self.assertEqual(z.get_month(months=-1, separator="*"),
                         z.get_month(months=-1, separator="*"))



__AMAP_API_KEY = "AMAP_API_KEY"


x_pi = 3.14159265358979324 * 3000.0 / 180.0
pi = 3.1415926535897932384626  # π
a = 6378245.0  # 长半轴
ee = 0.00669342162296594323  # 偏心率平方


def amap_geo_coding(address, city=None, api_key=None):
    import json
    import requests
    import traceback
    import warnings

    warnings.simplefilter("ignore", ResourceWarning)

    def get_api_key_from_params():
        try:
            from .zconfig import get_param
            return get_param(__AMAP_API_KEY)
        except:
            return None
    if not api_key:
        api_key = get_api_key_from_params()
    assert api_key, "缺失高德API KEY"

    try:
        response = requests.get(
            'https://restapi.amap.com/v3/geocode/geo',
            params={
                'key': api_key,
                'city': city,
                'address': address
            },
        )
    except Exception as ex:
        raise Exception("网络错误")

    try:
        result = json.loads(response.content.decode())
        assert result.get("status") == 1
        assert result.get("infocode") == "10000"
        assert result.get("info") == "OK"
        return result
    except Exception as ex:
        raise Exception("解析错误")


def gcj02_to_bd09(lng, lat):
    """
    火星坐标系(GCJ-02)转百度坐标系(BD-09)
    谷歌、高德——>百度
    :param lng:火星坐标经度
    :param lat:火星坐标纬度
    :return:
    """
    import math
    z = math.sqrt(lng * lng + lat * lat) + 0.00002 * math.sin(lat * x_pi)
    theta = math.atan2(lat, lng) + 0.000003 * math.cos(lng * x_pi)
    bd_lng = z * math.cos(theta) + 0.0065
    bd_lat = z * math.sin(theta) + 0.006
    return [bd_lng, bd_lat]


def bd09_to_gcj02(lng, lat):
    """
    百度坐标系(BD-09)转火星坐标系(GCJ-02)
    百度——>谷歌、高德
    :param bd_lat:百度坐标纬度
    :param bd_lon:百度坐标经度
    :return:转换后的坐标列表形式
    """
    import math
    x = lng - 0.0065
    y = lat - 0.006
    z = math.sqrt(x * x + y * y) - 0.00002 * math.sin(y * x_pi)
    theta = math.atan2(y, x) - 0.000003 * math.cos(x * x_pi)
    gg_lng = z * math.cos(theta)
    gg_lat = z * math.sin(theta)
    return [gg_lng, gg_lat]


def wgs84_to_gcj02(lng, lat):
    """
    WGS84转GCJ02(火星坐标系)
    :param lng:WGS84坐标系的经度
    :param lat:WGS84坐标系的纬度
    :return:
    """
    import math
    if out_of_china(lng, lat):  # 判断是否在国内
        return [lng, lat]
    dlat = _transformlat(lng - 105.0, lat - 35.0)
    dlng = _transformlng(lng - 105.0, lat - 35.0)
    radlat = lat / 180.0 * pi
    magic = math.sin(radlat)
    magic = 1 - ee * magic * magic
    sqrtmagic = math.sqrt(magic)
    dlat = (dlat * 180.0) / ((a * (1 - ee)) / (magic * sqrtmagic) * pi)
    dlng = (dlng * 180.0) / (a / sqrtmagic * math.cos(radlat) * pi)
    mglat = lat + dlat
    mglng = lng + dlng
    return [mglng, mglat]


def gcj02_to_wgs84(lng, lat):
    """
    GCJ02(火星坐标系)转GPS84
    :param lng:火星坐标系的经度
    :param lat:火星坐标系纬度
    :return:
    """
    import math
    if out_of_china(lng, lat):
        return [lng, lat]
    dlat = _transformlat(lng - 105.0, lat - 35.0)
    dlng = _transformlng(lng - 105.0, lat - 35.0)
    radlat = lat / 180.0 * pi
    magic = math.sin(radlat)
    magic = 1 - ee * magic * magic
    sqrtmagic = math.sqrt(magic)
    dlat = (dlat * 180.0) / ((a * (1 - ee)) / (magic * sqrtmagic) * pi)
    dlng = (dlng * 180.0) / (a / sqrtmagic * math.cos(radlat) * pi)
    mglat = lat + dlat
    mglng = lng + dlng
    return [lng * 2 - mglng, lat * 2 - mglat]


def bd09_to_wgs84(bd_lon, bd_lat):
    lon, lat = bd09_to_gcj02(bd_lon, bd_lat)
    return gcj02_to_wgs84(lon, lat)


def wgs84_to_bd09(lon, lat):
    lon, lat = wgs84_to_gcj02(lon, lat)
    return gcj02_to_bd09(lon, lat)


def _transformlat(lng, lat):
    import math
    ret = -100.0 + 2.0 * lng + 3.0 * lat + 0.2 * lat * lat + \
        0.1 * lng * lat + 0.2 * math.sqrt(math.fabs(lng))
    ret += (20.0 * math.sin(6.0 * lng * pi) + 20.0 *
            math.sin(2.0 * lng * pi)) * 2.0 / 3.0
    ret += (20.0 * math.sin(lat * pi) + 40.0 *
            math.sin(lat / 3.0 * pi)) * 2.0 / 3.0
    ret += (160.0 * math.sin(lat / 12.0 * pi) + 320 *
            math.sin(lat * pi / 30.0)) * 2.0 / 3.0
    return ret


def _transformlng(lng, lat):
    import math
    ret = 300.0 + lng + 2.0 * lat + 0.1 * lng * lng + \
        0.1 * lng * lat + 0.1 * math.sqrt(math.fabs(lng))
    ret += (20.0 * math.sin(6.0 * lng * pi) + 20.0 *
            math.sin(2.0 * lng * pi)) * 2.0 / 3.0
    ret += (20.0 * math.sin(lng * pi) + 40.0 *
            math.sin(lng / 3.0 * pi)) * 2.0 / 3.0
    ret += (150.0 * math.sin(lng / 12.0 * pi) + 300.0 *
            math.sin(lng / 30.0 * pi)) * 2.0 / 3.0
    return ret


def out_of_china(lng, lat):
    """
    判断是否在国内，不在国内不做偏移
    :param lng:
    :param lat:
    :return:
    """
    return not (lng > 73.66 and lng < 135.05 and lat > 3.86 and lat < 53.55)


def calculate_distance(loc1, loc2):
    """
    (lat1, lng1), (lat2, lng2)
    """
    from geopy.distance import geodesic
    from math import inf
    try:
        return geodesic(loc1, loc2).m
    except:
        return inf


if __name__ == '__main__':
    lng = 128.543
    lat = 37.065
    result1 = gcj02_to_bd09(lng, lat)
    result2 = bd09_to_gcj02(lng, lat)
    result3 = wgs84_to_gcj02(lng, lat)
    result4 = gcj02_to_wgs84(lng, lat)
    result5 = bd09_to_wgs84(lng, lat)
    result6 = wgs84_to_bd09(lng, lat)

    result7 = amap_geo_coding(address='北京市朝阳区朝阳公园',
                              api_key='API_KEY')  # 这里填写你的高德api的key
    print(result1, result2, result3, result4, result5, result6, result7)



import unittest


class ZconfigTests(unittest.TestCase):
    def setUp(self):
        import os
        super().setUp()
        self.root_dir = os.path.join("test", "file")
        from zzpy import init_dir
        init_dir(self.root_dir)

    def tearDown(self):
        from zzpy import remove_dir
        super().tearDown()
        remove_dir(self.root_dir)

    def test_get_param_none(self):
        from zzpy import get_param
        key = "UNITTEST_KEY"

        value = get_param(key)
        self.assertIsNone(value)

    def test_get_param_from_default_value(self):
        from zzpy import get_param
        key = "UNITTEST_KEY"
        default_value = "UNITTEST_DEFAULT"
        value = get_param(key, default_value=default_value)
        self.assertEqual(value, default_value)

    def test_get_param_from_default_config_value(self):
        from zzpy import get_param
        from zzpy import write_file
        import os
        import json

        key = "UNITTEST_KEY"

        default_config_path = os.path.join(
            self.root_dir, "default-config.json")
        default_config_value = "UNITTEST_DEFAULT_CONF"
        write_file(json.dumps(
            {key: default_config_value}), default_config_path)

        default_value = "UNITTEST_DEFAULT"

        value = get_param(key, default_value=default_value,
                          default_config_path=default_config_path, local_config_path=None)
        self.assertEqual(value, default_config_value)

    def test_get_param_from_local_config_value(self):
        from zzpy import get_param
        from zzpy import write_file
        import os
        import json

        key = "UNITTEST_KEY"

        local_config_path = os.path.join(self.root_dir, "local-config.json")
        local_config_value = "UNITTEST_LOCAL_CONF"
        write_file(json.dumps({key: local_config_value}), local_config_path)

        default_config_path = os.path.join(
            self.root_dir, "default-config.json")
        default_config_value = "UNITTEST_DEFAULT_CONF"
        write_file(json.dumps(
            {key: default_config_value}), default_config_path)

        default_value = "UNITTEST_DEFAULT"

        value = get_param(key, default_value=default_value,
                          default_config_path=default_config_path, local_config_path=local_config_path)
        self.assertEqual(value, local_config_value)

    def test_get_param_from_env(self):
        from zzpy import get_param
        from zzpy import write_file
        import os
        import json

        # init
        key = "UNITTEST_KEY"
        env_value = "UNITTEST_ENV"
        local_config_path = os.path.join(self.root_dir, "local-config.json")
        local_config_value = "UNITTEST_LOCAL_CONF"
        write_file(json.dumps({key: local_config_value}), local_config_path)

        default_config_path = os.path.join(
            self.root_dir, "default-config.json")
        default_config_value = "UNITTEST_DEFAULT_CONF"
        write_file(json.dumps(
            {key: default_config_value}), default_config_path)

        default_value = "UNITTEST_DEFAULT"

        os.environ[key] = env_value
        self.assertIn(key, os.environ)

        value = get_param(key, default_value=default_value,
                          default_config_path=default_config_path, local_config_path=local_config_path)
        self.assertEqual(value, env_value)

        # clean
        del os.environ[key]
        self.assertNotIn(key, os.environ)



__ES_URL_KEY = "ES_URL"

class EsConfig:
    default_host = "localhost"
    default_port = 9200

    def __init__(self, url=None):
        import re
        result = re.search(
            "^(es://){0,1}([\d\.\:|]+)[/\?]*(.*)$", url)
        groups = result.groups()
        self.hosts = groups[1].split("|")
        self.params = dict([it.split('=')
                            for it in groups[2].split('&')]) if groups[2] else {}
        self.http_auth = (self.params["user"], self.params["password"])


def es_connect(url=None, cert_path="/es-cert"):
    if not url:
        from .zconfig import get_param
        url = get_param(__ES_URL_KEY)
    assert url

    from elasticsearch import Elasticsearch
    import os

    conf = EsConfig(url)
    client = Elasticsearch(
        hosts=conf.hosts, http_auth=conf.http_auth, verify_certs=True)
    return client



import unittest
import datetime
import zzpy as z


class ZjsonJsondumpsTests(unittest.TestCase):
    def test_jsondumps_when_empty_dictionary(self):
        self.assertEqual(z.jsondumps({}), "{}")

    def test_jsondumps_when_empty_string(self):
        self.assertEqual(z.jsondumps(""), "\"\"")

    def test_jsondumps_when_empty_list(self):
        self.assertEqual(z.jsondumps([]), "[]")

    def test_jsondumps_when_contains_datetime(self):
        test_datetime_str = "2020-07-20 12:00:00"
        test_datetime_fmt = "%Y-%m-%d %H:%M:%S"
        d = datetime.datetime.strptime(
            test_datetime_str, test_datetime_fmt)
        expected_value = f'{{"a": 1, "d": "{test_datetime_str}"}}'
        self.assertEqual(z.jsondumps({"a": 1, "d": d}),
                         expected_value)

    def test_jsondumps_when_without_datetime(self):
        expected_value = f'{{"a": 1, "b": "2"}}'
        self.assertEqual(z.jsondumps({"a": 1, "b": "2"}),
                         expected_value)

    def test_jsondumps_with_indent(self):
        expected_value = '{\n    "k": "v"\n}'
        self.assertEqual(z.jsondumps({"k": "v"}, indent=4), expected_value)



"""系统（Python解释器系统；操作系统等）"""


def is_windows():
    from .zdefinition import SystemPlatform
    import sys
    return sys.platform == SystemPlatform.Windows


def windows_else(windows_value, else_value):
    return windows_value if is_windows() else else_value
